/**
 **	File ......... Button.cpp
 **	Published ....  2004-05-15
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <SDL.h>
#include "guitypedefs.h"
#include "guicolors.h"
#ifdef _WIN32
#pragma warning(push)

#include <yvals.h> // warning numbers get enabled in yvals.h 

#pragma warning(disable: 4251)  
#pragma warning(disable: 4786)  // identifier was truncated to 'number' characters in the debug information

// BUG: C4786 Warning Is Not Disabled with #pragma Warning
// STATUS: Microsoft has confirmed this to be a bug in the Microsoft product. This warning can be ignored.
// This occured only in the <map> container.

#include <sge.h>

#pragma warning(pop)
#else
#include <sge.h>
#endif
//#include <iotm/Mutex.h>
#include "SDL_prim.h"

#include "TTFont.h"
#include "SDLControl.h"
#include "Surface.h"
#include "Static.h"
#include "Button.h"


namespace gui
{


Button::Button(Surface *s,coord_t x,coord_t y,coord_t w,coord_t h,SurfaceHelper *pclHelper)
:Surface(s,x,y,w,h,pclHelper)
,m_screen_inverted(NULL)
,m_screen_inactive(NULL)
,m_style(0)
{
	SetBgColor(BUTTON_DEFAULT_BG_R,BUTTON_DEFAULT_BG_G,BUTTON_DEFAULT_BG_B);
	SetFgColor(BUTTON_DEFAULT_FG_R,BUTTON_DEFAULT_FG_G,BUTTON_DEFAULT_FG_B);
	SetFrameWidth(BUTTON_DEFAULT_FW);
	SetFrameColor(BUTTON_DEFAULT_FC_R,BUTTON_DEFAULT_FC_G,BUTTON_DEFAULT_FC_B);

	m_screen_inverted = CreateSDLSurface(true);
	m_screen_inactive = CreateSDLSurface(true);
}


Button::Button(Surface *s,coord_t x,coord_t y,buttonstyle_t st,SurfaceHelper *pclHelper)
:Surface(s,x,y,SBSIZE,SBSIZE,pclHelper)
,m_screen_inverted(NULL)
,m_screen_inactive(NULL)
,m_style(st)
{
	SetBgColor(BUTTON_DEFAULT_BG_R,BUTTON_DEFAULT_BG_G,BUTTON_DEFAULT_BG_B);
	SetFgColor(BUTTON_DEFAULT_FG_R,BUTTON_DEFAULT_FG_G,BUTTON_DEFAULT_FG_B);
	SetFrameWidth(BUTTON_DEFAULT_FW);
	SetFrameColor(BUTTON_DEFAULT_FC_R,BUTTON_DEFAULT_FC_G,BUTTON_DEFAULT_FC_B);

	m_screen_inverted = CreateSDLSurface(true);
	m_screen_inactive = CreateSDLSurface(true);
}


Button::~Button()
{
	if (m_screen_inverted)
	{
		SDL_FreeSurface(m_screen_inverted);
	}
	if (m_screen_inactive)
	{
		SDL_FreeSurface(m_screen_inactive);
	}
}


void Button::Draw()
{
	SDL_Rect *area = GetClientRectPtr();
//	color_t color = GetBgColor();
	TTFont *font = GetTTFont();
	SDL_Color fg;
	SDL_Color bg;
	coord_t x1;
	coord_t y1;
	coord_t x2;
	coord_t y2;
	coord_t x3;
	coord_t y3;
//	bool bMinneInverted = IsInverted();

//	SetInverted(false);

DEB(printf(" ******** Button redraw\n");)

	GetFgColor(fg);
	GetBgColor(bg);

#define SPACED 3

	switch (m_style)
	{
	case BUTTONSTYLE_SBUP: // Scrollbar
		x1 = area -> w / 2;
		y1 = SPACED;
		x2 = SPACED;
		y2 = area -> h - SPACED;
		x3 = area -> w - SPACED;
		y3 = area -> h - SPACED;
		// m_screen
		SDL_FillRect(m_screen, area, GetBgColor());
		SDL_fillTriangle(m_screen, x1,y1, x2,y2, x3,y3, GetFgColor());
		// m_screen_inverted
		SDL_FillRect(m_screen_inverted, area, GetFgColor());
		SDL_fillTriangle(m_screen_inverted, x1,y1, x2,y2, x3,y3, GetBgColor());
		// m_screen_inactive
		break;
	case BUTTONSTYLE_SBDOWN: // Scrollbar
		x1 = SPACED;
		y1 = SPACED;
		x2 = area -> w - SPACED;
		y2 = SPACED;
		x3 = area -> w / 2;
		y3 = area -> h - SPACED;
		// m_screen
		SDL_FillRect(m_screen, area, GetBgColor());
		SDL_fillTriangle(m_screen, x1,y1, x2,y2, x3,y3, GetFgColor());
		// m_screen_inverted
		SDL_FillRect(m_screen_inverted, area, GetFgColor());
		SDL_fillTriangle(m_screen_inverted, x1,y1, x2,y2, x3,y3, GetBgColor());
		// m_screen_inactive
		break;
	case BUTTONSTYLE_SBRIGHT: // Scrollbar
		x1 = SPACED;
		y1 = SPACED;
		x2 = area -> w - SPACED;
		y2 = area -> h / 2;
		x3 = SPACED;
		y3 = area -> h - SPACED;
		// m_screen
		SDL_FillRect(m_screen, area, GetBgColor());
		SDL_fillTriangle(m_screen, x1,y1, x2,y2, x3,y3, GetFgColor());
		// m_screen_inverted
		SDL_FillRect(m_screen_inverted, area, GetFgColor());
		SDL_fillTriangle(m_screen_inverted, x1,y1, x2,y2, x3,y3, GetBgColor());
		// m_screen_inactive
		break;
	case BUTTONSTYLE_SBLEFT: // Scrollbar
		x1 = SPACED;
		y1 = area -> h / 2;
		x2 = area -> w - SPACED;
		y2 = SPACED;
		x3 = area -> w - SPACED;
		y3 = area -> h - SPACED;
		// m_screen
		SDL_FillRect(m_screen, area, GetBgColor());
		SDL_fillTriangle(m_screen, x1,y1, x2,y2, x3,y3, GetFgColor());
		// m_screen_inverted
		SDL_FillRect(m_screen, area, GetFgColor());
		SDL_fillTriangle(m_screen, x1,y1, x2,y2, x3,y3, GetBgColor());
		// m_screen_inactive
		break;

	default:
	{
	// Normal
	SDL_FillRect(m_screen, area, GetBgColor());

	if (font && font -> GetFont())
	{
		SDL_Rect rect = sge_TTF_TextSize(font -> GetFont(), (char *)GetText());
		coord_t x = GetW() / 2 - rect.w / 2;
		coord_t y = GetH() / 2 + font -> GetH() / 2;

		sge_tt_textoutf(m_screen, font -> GetFont(),
			x,y,
			fg.r,fg.g,fg.b,
			bg.r,bg.g,bg.b,
			SDL_ALPHA_OPAQUE,
			"%s",GetText() );
	}
	DrawFrame(m_screen, GetFrameWidth(), GetFrameColor());

	// Inverted
	SDL_FillRect(m_screen_inverted, area, GetFgColor());

	if (font && font -> GetFont())
	{
		SDL_Rect rect = sge_TTF_TextSize(font -> GetFont(), (char *)GetText());
		coord_t x = GetW() / 2 - rect.w / 2;
		coord_t y = GetH() / 2 + font -> GetH() / 2;

		sge_tt_textoutf(m_screen_inverted, font -> GetFont(),
			x,y,
			bg.r,bg.g,bg.b,
			fg.r,fg.g,fg.b,
			SDL_ALPHA_OPAQUE,
			"%s",GetText() );
	}
	DrawFrame(m_screen_inverted, GetFrameWidth(), GetFrameColor());
	
	// Inactive
	} // case 0
		break;
	} // switch (m_style)
//	SetInverted(bMinneInverted);
}


SDL_Surface *Button::GetScreen()
{
	if (IsInverted())
	{
		return m_screen_inverted;
	}
/*
	if (IsInactive())
	{
		return m_screen_inactive;
	}
*/
	return m_screen;
}


} // namespace

//sge_TTF_FontHeight 
//sge_TTF_TextSize
