/**
 **	File ......... PopupMenuItem.cpp
 **	Published ....  2004-05-15
 **	Author ....... grymse@alhem.net
**/
/*
Copyright (C) 2004  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <SDL.h>
#include "guitypedefs.h"
#include "guicolors.h"
#ifdef _WIN32
#pragma warning(push)

#include <yvals.h> // warning numbers get enabled in yvals.h 

#pragma warning(disable: 4251)  
#pragma warning(disable: 4786)  // identifier was truncated to 'number' characters in the debug information

// BUG: C4786 Warning Is Not Disabled with #pragma Warning
// STATUS: Microsoft has confirmed this to be a bug in the Microsoft product. This warning can be ignored.
// This occured only in the <map> container.

#include <sge.h>

#pragma warning(pop)
#else
#include <sge.h>
#endif
//#include <iotm/Mutex.h>
#include "SDL_prim.h"

#include "Surface.h"
#include "MenuItem.h"
#include "Menu.h"
#include "SDLControl.h"
#include "PopupMenuItem.h"
#include "Xml.h"

using util::Xml;


namespace gui
{


PopupMenuItem::PopupMenuItem(Surface *s,const string &str,TTFont *font,int id,SurfaceHelper *pclHelper)
:MenuItem(s,str,font,pclHelper)
,m_id(id)
,m_pclSubmenu(NULL)
{
}


PopupMenuItem::~PopupMenuItem()
{
	if (m_pclSubmenu)
	{
		delete m_pclSubmenu;
	}
}


void PopupMenuItem::OnClick(coord_t xx,coord_t yy)
{
	Menu *pclMenu = static_cast<Menu *>(GetParent());
	Surface *pclParent = pclMenu -> GetParent();
	coord_t x = pclMenu -> GetX() + pclMenu -> GetW();
	coord_t y = pclMenu -> GetY() + GetY();

DEB(printf("PopupMenuItem::OnClick()\n");)

	if (!m_pclSubmenu)
	{
		m_pclSubmenu = pclMenu -> GetSubMenu(m_id);
		if (!m_pclSubmenu)
		{
DEB(printf(" * m_pclSubmenu NULL\n");)
			return;
		}
		if (x + m_pclSubmenu -> GetW() >= GetSDLControl() -> GetScreen() -> w ||
			y + m_pclSubmenu -> GetH() >= GetSDLControl() -> GetScreen() -> h)
		{
DEB(printf(" * m_pclSubmenu offscreen\n");)
			delete m_pclSubmenu;
			m_pclSubmenu = NULL;
			return;
		}
		else
		{
DEB(printf(" * m_pclSubmenu created\n");)
			pclParent -> AddChild(m_pclSubmenu);
		}
	}
	else
	{
DEB(printf(" * m_pclSubmenu exists\n");)
	}
	m_pclSubmenu -> SetX(x);
	m_pclSubmenu -> SetY(y);
	pclParent -> CalculateAbsolute();
	m_pclSubmenu -> InvalidateEv();
	pclMenu -> ActivateSubMenu(m_pclSubmenu, x, y);
}


void PopupMenuItem::Draw()
{
	SDL_Rect *a = GetClientRectPtr();

	MenuItem::Draw();

#define DIST 3

	coord_t x1 = a -> x + a -> w - a -> h + DIST;
	coord_t y1 = DIST;
	coord_t x2 = a -> x + a -> w - DIST;
	coord_t y2 = a -> h / 2;
	coord_t x3 = x1;
	coord_t y3 = a -> h - DIST;

	SDL_fillTriangle(GetScreen(), x1,y1, x2,y2, x3,y3, GetFgColor());
}


void PopupMenuItem::DumpMembers(FILE *fil)
{
	Xml xml(fil);
	
	MenuItem::DumpMembers(fil);
	
	xml.Write("POPUPMENUID", (long)m_id);
}


} // namespace

